using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;

namespace Components.Calendar
{
    /// <summary>
    /// Classe de base pour la gestion de la liste des rendez-vous
    /// </summary>
    [Serializable]
    public class RootMeetingSet : IEnumerator, IEnumerable, IMeetingRowList
    {
        #region Variables
        /// <summary>
        /// Index pour l'numration
        /// </summary>
        private int Index = -1;
        /// <summary>
        /// Indicates the biggest ID of the list
        /// </summary>
        private int vMaxID = -1;
        /// <summary>
        /// Liste de tous les rendez-vous
        /// </summary>
        protected List<MeetingRow> vMeetingList = new List<MeetingRow>();
        #endregion

        #region Constructor
        /// <summary>
        /// Constructeur
        /// </summary>
        public RootMeetingSet() { }
        /// <summary>
        /// Constructeur
        /// </summary>
        /// <param name="_MeetingList">Liste des rendez-vous pour une priode donne</param>
        public RootMeetingSet(List<MeetingRow> _MeetingList) { vMeetingList = _MeetingList; }
        #endregion

        #region Methods
        /// <summary>
        /// Default method for MeetingSet filling (Do Nothing)
        /// </summary>
        public virtual void Fill() { }
        /// <summary>
        /// Fill the current instance of the meeting set with an implementation of IMeetingRowList
        /// </summary>
        /// <param name="_Mrl"></param>
        public void Fill(IMeetingRowList _Mrl)
        {
            this.MeetingList = _Mrl.MeetingList;
        }
        /// <summary>
        /// Fait une recherche du nombre de rendez-vous  la date donne
        /// </summary>
        /// <param name="_Year">Anne du rendez-vous</param>
        /// <param name="_Month">Mois du rendez-vous</param>
        /// <returns>Une liste de DateToDisplay contenant les rendez-vous</returns>
        public List<MeetingRow> Seek(int _Year, int _Month)
        {
            if (vMeetingList == null) return new List<MeetingRow>();

            List<MeetingRow> mr = new List<MeetingRow>();

            foreach (MeetingRow Row in vMeetingList)
            {
                if (Row.StartingDate.Year == _Year)
                {
                    if (Row.StartingDate.Month == _Month)
                        mr.Add(Row);
                }
            }
            return mr;
        }
        /// <summary>
        /// Fait une recherche du nombre de rendez-vous  la date donne
        /// </summary>
        /// <param name="_Year">Anne du rendez-vous</param>
        /// <param name="_Month">Mois du rendez-vous</param>
        /// <param name="_Day">Jour du rendez-vous</param>
        /// <returns>Une liste de DateToDisplay contenant les rendez-vous</returns>
        public List<MeetingRow> Seek(int _Year, int _Month, int _Day)
        {
            if (vMeetingList == null) return new List<MeetingRow>();

            List<MeetingRow> mr = new List<MeetingRow>();

            foreach (MeetingRow Row in vMeetingList)
            {
                if (Row.StartingDate.Year == _Year)
                {
                    if (Row.StartingDate.Month == _Month)
                    {
                        if (Row.StartingDate.Day == _Day)
                            mr.Add(Row);
                    }
                }
            }
            return mr;
        }
        /// <summary>
        /// Recherche un rendez-vous  une date prcise (Heure comprise)
        /// </summary>
        /// <param name="_FullDate">DateToDisplay du rendez-vous</param>
        /// <returns>Une liste de DateToDisplay contenant les rendez-vous</returns>
        public List<MeetingRow> Seek(DateTime _FullDate)
        {
            if (vMeetingList == null) return new List<MeetingRow>();

            List<MeetingRow> mr = new List<MeetingRow>();

            foreach (MeetingRow Row in vMeetingList)
            {
                if (Row.StartingDate.Year == _FullDate.Year
                    && Row.StartingDate.Month == _FullDate.Month
                    && Row.StartingDate.Day == _FullDate.Day)
                    mr.Add(Row);
            }
            return mr;
        }
        /// <summary>
        /// Retourne le nombre de rendez-vous  la date donne
        /// </summary>
        /// <param name="_Year">Anne du rendez-vous</param>
        /// <param name="_Month">Mois du rendez-vous</param>
        /// <returns>Le nombre de rendez-vous  cette date</returns>
        public int CountElementAt(int _Year, int _Month)
        {
            int Count = 0;

            foreach (MeetingRow mr in vMeetingList)
            {
                if (mr.StartingDate.Year == _Year
                    && mr.StartingDate.Month == _Month)
                    Count++;
            }
            return Count;
        }
        /// <summary>
        /// Retourne le nombre de rendez-vous  la date donne
        /// </summary>
        /// <param name="_Year">Anne du rendez-vous</param>
        /// <param name="_Month">Mois du rendez-vous</param>
        /// <param name="_Day">Jours du rendez-vous</param>
        /// <returns>Le nombre de rendez-vous  cette date</returns>
        public int CountElementAt(int _Year, int _Month, int _Day)
        {
            int Count = 0;

            foreach (MeetingRow mr in vMeetingList)
            {
                if (mr.StartingDate.Year == _Year
                    && mr.StartingDate.Month == _Month
                    && mr.StartingDate.Day == _Day)
                    Count++;
            }
            return Count;
        }
        /// <summary>
        /// Ajoute un lment dans le liste des rendez-vous
        /// </summary>
        /// <param name="_Value">Un rendezv-vous</param>
        /// <returns>Un identifiant de rendez-vous (son index dans la liste)</returns>
        /// <remarks>Cet mthode est virtuelle parce que le code d'insertion dans
        /// La base de donne ou tout autre systme de sauvegarde sur fichier
        /// doit tre implment dans la classe fille</remarks>
        public virtual void Add(MeetingRow _Value)
        {
            MeetingRow value = (MeetingRow)_Value.Clone();
            value.SetID(++this.vMaxID);
            vMeetingList.Add(value);
            this.Sort();
            //return vMeetingList.IndexOf(_Value);
        }
        /// <summary>
        /// Supprime un rendez-vous  la date donne dans le liste des rendez-vous
        /// </summary>
        /// <param name="_Value">Le rendez-vous  supprimer</param>
        /// <remarks>N'ffectue que une seule suppression</remarks>
        public virtual void Remove(MeetingRow _Value)
        {
            foreach (MeetingRow row in vMeetingList)
            {
                if (row.HasSameContents(_Value))
                {
                    this.vMeetingList.Remove(row);
                    this.vMaxID = -1;
                    for (int i = 0; i < this.vMeetingList.Count; i++)
                        vMeetingList[i].SetID(++vMaxID);
                    break;
                }
            }
        }
        /// <summary>
        /// Fait une modification du rendez-vous dsign par _mr.MeetingData
        /// </summary>
        /// <param name="_mr">DateToDisplay contenant les informations du nouveau rendez-vous</param>        
        public virtual void Modify(MeetingRow _mr)
        {
            for (int i = 0; i < vMeetingList.Count; i++)
            {
                if (vMeetingList[i].HasSameContents(_mr))
                {
                    vMeetingList[i] = _mr;
                    break;
                }
            }
        }
        /// <summary>
        /// Mthode de triage des rendez-vous par date en ordre croissant
        /// </summary>
        /// <remarks>Utilisation du Bubble Sort</remarks>
        public void Sort()
        {
            int elm = vMeetingList.Count;
            MeetingRow buf;

            for (int i = (elm - 1); i >= 1; i--)
            {
                for (int j = i; j <= i; j++)
                {
                    if (vMeetingList[j - 1].StartingDate > vMeetingList[j].StartingDate)
                    {
                        buf = vMeetingList[j - 1];
                        vMeetingList[j - 1] = vMeetingList[j];
                        vMeetingList[j] = buf;
                    }
                }
            }
        }
        #endregion

        #region Properties
        /// <summary>
        /// Retourne l'lment  l'index donn
        /// </summary>
        /// <param name="_index">index qu'il faut retourner</param>
        /// <returns>Le rendez-vous  l'index donn</returns>
        /// <exception cref="ArgumentOutOfRangeException">En cas d'index hors des limites</exception>
        public MeetingRow this[int _index]
        {
            get
            {
                try { return vMeetingList[_index]; }
                catch (ArgumentOutOfRangeException) { throw; }
            }
        }
        /// <summary>
        /// Retourne le nombre de rendez-vous dans la liste
        /// </summary>
        public int Count { get { return vMeetingList.Count; } }
        /// <summary>
        /// Retourne ou initialise une List contenant les rendez-vous
        /// </summary>
        public List<MeetingRow> MeetingList
        {
            get { return this.vMeetingList; }
            set { vMeetingList = value; }
        }
        #endregion

        #region IEnumerator Members
        /// <summary>
        /// Retourne l'lment courrant
        /// </summary>
        public object Current { get { return vMeetingList[Index]; } }
        /// <summary>
        /// Vrifie si il reste un lment suivant
        /// </summary>
        /// <returns></returns>
        public bool MoveNext()
        {
            Index++;
            return (Index < vMeetingList.Count);
        }
        /// <summary>
        /// Rinitialise l'index de la liste
        /// </summary>
        public void Reset() { Index = -1; }

        #endregion

        #region IEnumerable Members
        /// <summary>
        /// Returns an enumerator that iterates through a collection.
        /// </summary> An System.Collections.IEnumerator object that can be used to iterate through
        /// the collection.
        public IEnumerator GetEnumerator() { Index = -1; return (IEnumerator)this; }

        #endregion

        #region IMeetingRowList Members
        /// <summary>
        /// Gets an IMeetingRowList from a DateToDisplay list
        /// </summary>
        /// <param name="_List">List of DateToDisplay</param>
        /// <returns>An instance of a IMeetingRowList</returns>
        public void Fill(List<MeetingRow> _List)
        {
            this.vMeetingList = _List;
            IMeetingRowList mr = this as IMeetingRowList;
        }
        #endregion

        #region Operator
        /// <summary>
        /// Fait une convertion d'un RootMeetingSet vers une liste de rendez-vous
        /// </summary>
        /// <param name="_RootMeetingSet">Le RootMeetingSet  convertir</param>
        /// <returns>Une List(DateToDisplay) contenue dans le RootMeetingSet</returns>
        public static implicit operator List<MeetingRow>(RootMeetingSet _RootMeetingSet)
        {
            return _RootMeetingSet.vMeetingList;
        }
        #endregion
    }
}
